﻿using k8s;
using System;
using System.Text;

namespace k8sutil
{
    class Program
    {
        static void Main(string[] args)
        {
            string path = Environment.ExpandEnvironmentVariables(@"%USERPROFILE%\.kube\config");
            KubernetesClientConfiguration config = KubernetesClientConfiguration.BuildConfigFromConfigFile(path);
            // 또는, KubernetesClientConfiguration config = KubernetesClientConfiguration.BuildDefaultConfig();

            IKubernetes client = new Kubernetes(config);

            ListSecrets(client);

            Console.WriteLine();

            /* 혹은, secrets의 이름과 네임스페이스를 알고 있다면 곧바로 쿼리

c:\temp> kubectl get secrets
NAME                  TYPE                                  DATA   AGE
default-token-qnwbl   kubernetes.io/service-account-token   3      50d
            */

            // var v1secret = client.ReadNamespacedSecret("default-token-qnwbl", "default");

            string secretName = client.ListNamespacedSecret("default").Items[0].Metadata.Name;
            string namespaceProperty = "default";
            var v1secret = client.ReadNamespacedSecret(secretName, namespaceProperty);
            foreach (var data in v1secret.Data)
            {
                Console.WriteLine($"\t{data.Key} == {data.Value.Length} byte(s)");
            }
        }

        private static void ListSecrets(IKubernetes client)
        {
            var list = client.ListNamespacedSecret("default");
            foreach (var item in list.Items)
            {
                Console.WriteLine($"{nameof(item.ApiVersion)}: {item.ApiVersion}"); // (null)
                Console.WriteLine($"{nameof(item.StringData)}: {item.StringData}"); // (null)

                Console.WriteLine($"{nameof(item.Type)}: {item.Type}"); // kubernetes.io/service-account-token
                Console.WriteLine($"{nameof(item.Kind)}: {item.Kind}"); // (null)

                Console.WriteLine($"{nameof(item.Immutable)}: {item.Immutable}"); // (null)

                Console.WriteLine();

                Console.WriteLine("=== V1ObjectMetadata ===");
                Console.WriteLine($"\t{nameof(item.Metadata.Name)}: {item.Metadata.Name}"); // default-token-qnwbl
                Console.WriteLine($"\t{nameof(item.Metadata.ClusterName)}: {item.Metadata.ClusterName}"); // (null)
                Console.WriteLine($"\t{nameof(item.Metadata.CreationTimestamp)}: {item.Metadata.CreationTimestamp}"); // 2021-01-25 오전 6:01:59
                Console.WriteLine($"\t{nameof(item.Metadata.DeletionGracePeriodSeconds)}: {item.Metadata.DeletionGracePeriodSeconds}"); // (null)
                Console.WriteLine($"\t{nameof(item.Metadata.GenerateName)}: {item.Metadata.GenerateName}"); // (null)
                Console.WriteLine($"\t{nameof(item.Metadata.Generation)}: {item.Metadata.Generation}"); // (null)
                Console.WriteLine($"\t{nameof(item.Metadata.NamespaceProperty)}: {item.Metadata.NamespaceProperty}"); // default
                Console.WriteLine($"\t{nameof(item.Metadata.SelfLink)}: {item.Metadata.SelfLink}"); // /api/v1/namespaces/default/secrets/default-token-qnwbl
                Console.WriteLine($"\t{nameof(item.Metadata.Uid)}: {item.Metadata.Uid}"); // 6c584cda-d9a9-4b5c-8489-7e4365866014

                Console.WriteLine();

                Console.WriteLine($"=== Annotations ({item.Metadata.Annotations.Count}) ===");
                foreach (var (k, v) in item.Metadata.Annotations)
                {
                    Console.WriteLine($"\t{k} == {v}");
                }

                Console.WriteLine();

                Console.WriteLine($"=== Labels ({item.Metadata.Labels?.Count}) ===");
                if (item.Metadata.Labels != null)
                {
                    foreach (var (k, v) in item.Metadata.Labels)
                    {
                        Console.WriteLine($"\t{k} == {v}");
                    }
                }

                Console.WriteLine();

                Console.WriteLine($"=== OwnerReferences ({item.Metadata.OwnerReferences?.Count}) ===");
                if (item.Metadata.OwnerReferences != null)
                {
                    foreach (var data in item.Metadata.OwnerReferences)
                    {
                        PrintOwnerReference(data);
                    }
                }

                Console.WriteLine();

                Console.WriteLine($"=== ManagedFields ({item.Metadata.ManagedFields?.Count}) ===");
                if (item.Metadata.ManagedFields != null)
                {
                    foreach (var data in item.Metadata.ManagedFields)
                    {
                        Console.WriteLine($"\t{nameof(data.ApiVersion)}: {data.ApiVersion}"); // v1
                        Console.WriteLine($"\t{nameof(data.FieldsType)}: {data.FieldsType}"); // FieldsV1
                        Console.WriteLine($"\t{nameof(data.FieldsV1)}: {data.FieldsV1}"); // ...json...
                        /*
                                                                        {
                                                                          "f:data": {
                                                                            ".": {},
                                                                            "f:ca.crt": {},
                                                                            "f:namespace": {},
                                                                            "f:token": {}
                                                                          },
                                                                          "f:metadata": {
                                                                            "f:annotations": {
                                                                              ".": {},
                                                                              "f:kubernetes.io/service-account.name": {},
                                                                              "f:kubernetes.io/service-account.uid": {}
                                                                            }
                                                                          },
                                                                          "f:type": {}
                                                                        }
                        */
                        Console.WriteLine($"\t{nameof(data.Operation)}: {data.Operation}"); // Update
                        Console.WriteLine($"\t{nameof(data.Manager)}: {data.Manager}"); // Kube-controller-manager
                        Console.WriteLine($"\t{nameof(data.Time)}: {data.Time}"); // v1
                    }
                }

                Console.WriteLine("=== Data ===");
                /*
                ca.crt == byte []
                namespace == byte [] ("default")
                token == byte []
                */
                foreach (var (k, v) in item.Data)
                {
                    Console.WriteLine($"\t{k} == {PrintValue(v)}");
                }

            }
        }

        private static void PrintOwnerReference(k8s.Models.V1OwnerReference data)
        {
            Console.WriteLine($"\t{nameof(data.ApiVersion)} == {data.ApiVersion}");
            Console.WriteLine($"\t{nameof(data.BlockOwnerDeletion)} == {data.BlockOwnerDeletion}");
            Console.WriteLine($"\t{nameof(data.Controller)} == {data.Controller}");
            Console.WriteLine($"\t{nameof(data.Kind)} == {data.Kind}");
            Console.WriteLine($"\t{nameof(data.Name)} == {data.Name}");
            Console.WriteLine($"\t{nameof(data.Uid)} == {data.Uid}");
        }

        private static string PrintValue(byte[] value)
        {
            return Encoding.UTF8.GetString(value);
        }
    }
}
